@@ -23,20 +23,24 @@ from sys import platform as _platform
 class QrackSystem:
     def __init__(self):
         shared_lib_path = ""
-        if os.environ.get('PYQRACK_SHARED_LIB_PATH') != None:
-            shared_lib_path = os.environ.get('PYQRACK_SHARED_LIB_PATH')
+        if os.environ.get("PYQRACK_SHARED_LIB_PATH") != None:
+            shared_lib_path = os.environ.get("PYQRACK_SHARED_LIB_PATH")
         elif _platform == "win32":
             shared_lib_path = os.path.dirname(__file__) + "/qrack_lib/qrack_pinvoke.dll"
         elif _platform == "darwin":
-            shared_lib_path = os.path.dirname(__file__) + "/qrack_lib/libqrack_pinvoke.dylib"
+            shared_lib_path = (
+                os.path.dirname(__file__) + "/qrack_lib/libqrack_pinvoke.dylib"
+            )
         else:
-            shared_lib_path = os.path.dirname(__file__) + "/qrack_lib/libqrack_pinvoke.so"
+            shared_lib_path = (
+                os.path.dirname(__file__) + "/qrack_lib/libqrack_pinvoke.so"
+            )
 
         try:
             self.qrack_lib = CDLL(shared_lib_path)
         except Exception as e:
             if _platform == "win32":
-                shared_lib_path = "C:/Program Files/libqrack*/lib/qrack_pinvoke.lib"
+                shared_lib_path = "C:/Program Files (x86)/Qrack/bin/qrack_pinvoke.dll"
             elif _platform == "darwin":
                 shared_lib_path = "/usr/local/lib/qrack/libqrack_pinvoke.dylib"
             else:
@@ -46,7 +50,9 @@ class QrackSystem:
                 self.qrack_lib = CDLL(shared_lib_path)
             except Exception as e:
                 if _platform == "win32":
-                    shared_lib_path = "C:/Program Files (x86)/libqrack*/lib/qrack_pinvoke.lib"
+                    shared_lib_path = (
+                        "C:/Program Files (x86)/Qrack/bin/qrack_pinvoke.dll"
+                    )
                 elif _platform == "darwin":
                     shared_lib_path = "/usr/lib/qrack/libqrack_pinvoke.dylib"
                 else:
@@ -55,12 +61,14 @@ class QrackSystem:
                 try:
                     self.qrack_lib = CDLL(shared_lib_path)
                 except Exception as e:
-                    print("IMPORTANT: Did you remember to install OpenCL, if your Qrack version was built with OpenCL?")
+                    print(
+                        "IMPORTANT: Did you remember to install OpenCL, if your Qrack version was built with OpenCL?"
+                    )
                     raise e
 
         self.fppow = 5
         if "QRACK_FPPOW" in os.environ:
-            self.fppow = int(os.environ.get('QRACK_FPPOW'))
+            self.fppow = int(os.environ.get("QRACK_FPPOW"))
         if self.fppow < 4 or self.fppow > 7:
             raise ValueError(
                 "QRACK_FPPOW environment variable must be an integer >3 and <8. (Qrack builds from 4 for fp16/half, up to 7 for fp128/quad."
@@ -76,22 +84,25 @@ class QrackSystem:
         self.qrack_lib.Dump.restype = None
         self.qrack_lib.Dump.argtypes = [
             c_ulonglong,
-            CFUNCTYPE(c_ulonglong, c_double, c_double)
+            CFUNCTYPE(c_ulonglong, c_double, c_double),
         ]
 
-        # These next two methods need to have c_double pointers, if PyQrack is built with fp64.
+        # These next few methods need to have c_double pointers, if PyQrack is built with fp64.
         self.qrack_lib.InKet.restype = None
         self.qrack_lib.OutKet.restype = None
         self.qrack_lib.OutProbs.restype = None
+        self.qrack_lib.OutReducedDensityMatrix.restype = None
 
         if self.fppow < 6:
             self.qrack_lib.InKet.argtypes = [c_ulonglong, POINTER(c_float)]
             self.qrack_lib.OutKet.argtypes = [c_ulonglong, POINTER(c_float)]
             self.qrack_lib.OutProbs.argtypes = [c_ulonglong, POINTER(c_float)]
+            self.qrack_lib.OutReducedDensityMatrix.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong), POINTER(c_float)]
         else:
             self.qrack_lib.InKet.argtypes = [c_ulonglong, POINTER(c_double)]
             self.qrack_lib.OutKet.argtypes = [c_ulonglong, POINTER(c_double)]
             self.qrack_lib.OutProbs.argtypes = [c_ulonglong, POINTER(c_double)]
+            self.qrack_lib.OutReducedDensityMatrix.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong), POINTER(c_double)]
 
         self.qrack_lib.init.restype = c_ulonglong
         self.qrack_lib.init.argtypes = []
@@ -117,9 +128,12 @@ class QrackSystem:
             c_bool,
             c_bool,
             c_bool,
-            c_bool
+            c_bool,
         ]
 
+        self.qrack_lib.init_count_stabilizer.restype = c_ulonglong
+        self.qrack_lib.init_count_stabilizer.argtypes = [c_ulonglong]
+
         self.qrack_lib.init_clone.restype = c_ulonglong
         self.qrack_lib.init_clone.argtypes = [c_ulonglong]
 
@@ -132,22 +146,35 @@ class QrackSystem:
         self.qrack_lib.set_concurrency.restype = None
         self.qrack_lib.set_concurrency.argtypes = [c_ulonglong, c_ulonglong]
 
+        self.qrack_lib.set_device.restype = None
+        self.qrack_lib.set_device.argtypes = [c_ulonglong, c_longlong]
+
+        self.qrack_lib.set_device_list.restype = None
+        self.qrack_lib.set_device_list.argtypes = [
+            c_ulonglong,
+            c_ulonglong,
+            POINTER(c_longlong),
+        ]
+
         # pseudo-quantum
 
+        self.qrack_lib.HighestProbAll.restype = None
+        self.qrack_lib.HighestProbAll.argtypes = [c_ulonglong, POINTER(c_ulonglong)]
+
         self.qrack_lib.ProbAll.restype = None
         if self.fppow == 5:
             self.qrack_lib.ProbAll.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
         elif self.fppow == 6:
             self.qrack_lib.ProbAll.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
 
         self.qrack_lib.Prob.restype = c_double
@@ -161,7 +188,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_bool)
+            POINTER(c_bool),
         ]
 
         self.qrack_lib.PermutationProbRdm.restype = c_double
@@ -170,14 +197,14 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             POINTER(c_bool),
-            c_bool
+            c_bool,
         ]
 
         self.qrack_lib.PermutationExpectation.restype = c_double
         self.qrack_lib.PermutationExpectation.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.PermutationExpectationRdm.restype = c_double
@@ -185,7 +212,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_bool
+            c_bool,
         ]
 
         self.qrack_lib.FactorizedExpectation.restype = c_double
@@ -194,7 +221,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.FactorizedExpectationRdm.restype = c_double
@@ -204,7 +231,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_bool
+            c_bool,
         ]
 
         if self.fppow == 5:
@@ -213,7 +240,7 @@ class QrackSystem:
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.FactorizedExpectationFpRdm.restype = c_double
             self.qrack_lib.FactorizedExpectationFpRdm.argtypes = [
@@ -221,21 +248,21 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                c_bool
+                c_bool,
             ]
             self.qrack_lib.UnitaryExpectation.restype = c_double
             self.qrack_lib.UnitaryExpectation.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.MatrixExpectation.restype = c_double
             self.qrack_lib.MatrixExpectation.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.UnitaryExpectationEigenVal.restype = c_double
             self.qrack_lib.UnitaryExpectationEigenVal.argtypes = [
@@ -243,7 +270,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.MatrixExpectationEigenVal.restype = c_double
             self.qrack_lib.MatrixExpectationEigenVal.argtypes = [
@@ -251,7 +278,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
         elif self.fppow == 6:
             self.qrack_lib.FactorizedExpectationFp.restype = c_double
@@ -259,7 +286,7 @@ class QrackSystem:
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.FactorizedExpectationFpRdm.restype = c_double
             self.qrack_lib.FactorizedExpectationFpRdm.argtypes = [
@@ -267,21 +294,21 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                c_bool
+                c_bool,
             ]
             self.qrack_lib.UnitaryExpectation.restype = c_double
             self.qrack_lib.UnitaryExpectation.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.MatrixExpectation.restype = c_double
             self.qrack_lib.MatrixExpectation.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.UnitaryExpectationEigenVal.restype = c_double
             self.qrack_lib.UnitaryExpectationEigenVal.argtypes = [
@@ -289,7 +316,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.MatrixExpectationEigenVal.restype = c_double
             self.qrack_lib.MatrixExpectationEigenVal.argtypes = [
@@ -297,7 +324,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
 
         self.qrack_lib.PauliExpectation.restype = c_double
@@ -305,14 +332,14 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.Variance.restype = c_double
         self.qrack_lib.Variance.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.VarianceRdm.restype = c_double
@@ -320,7 +347,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_bool
+            c_bool,
         ]
 
         self.qrack_lib.FactorizedVariance.restype = c_double
@@ -329,7 +356,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.FactorizedVarianceRdm.restype = c_double
@@ -339,7 +366,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_bool
+            c_bool,
         ]
 
         if self.fppow == 5:
@@ -348,7 +375,7 @@ class QrackSystem:
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.FactorizedVarianceFpRdm.restype = c_double
             self.qrack_lib.FactorizedVarianceFpRdm.argtypes = [
@@ -356,21 +383,21 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                c_bool
+                c_bool,
             ]
             self.qrack_lib.UnitaryVariance.restype = c_double
             self.qrack_lib.UnitaryVariance.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.MatrixVariance.restype = c_double
             self.qrack_lib.MatrixVariance.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.UnitaryVarianceEigenVal.restype = c_double
             self.qrack_lib.UnitaryVarianceEigenVal.argtypes = [
@@ -378,7 +405,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
             self.qrack_lib.MatrixVarianceEigenVal.restype = c_double
             self.qrack_lib.MatrixVarianceEigenVal.argtypes = [
@@ -386,7 +413,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_float),
-                POINTER(c_float)
+                POINTER(c_float),
             ]
         elif self.fppow == 6:
             self.qrack_lib.FactorizedVarianceFp.restype = c_double
@@ -394,7 +421,7 @@ class QrackSystem:
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.FactorizedVarianceFpRdm.restype = c_double
             self.qrack_lib.FactorizedVarianceFpRdm.argtypes = [
@@ -402,21 +429,21 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                c_bool
+                c_bool,
             ]
             self.qrack_lib.UnitaryVariance.restype = c_double
             self.qrack_lib.UnitaryVariance.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.MatrixVariance.restype = c_double
             self.qrack_lib.MatrixVariance.argtypes = [
                 c_ulonglong,
                 c_ulonglong,
                 POINTER(c_ulonglong),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.UnitaryVarianceEigenVal.restype = c_double
             self.qrack_lib.UnitaryVarianceEigenVal.argtypes = [
@@ -424,7 +451,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
             self.qrack_lib.MatrixVarianceEigenVal.restype = c_double
             self.qrack_lib.MatrixVarianceEigenVal.argtypes = [
@@ -432,7 +459,7 @@ class QrackSystem:
                 c_ulonglong,
                 POINTER(c_ulonglong),
                 POINTER(c_double),
-                POINTER(c_double)
+                POINTER(c_double),
             ]
 
         self.qrack_lib.PauliVariance.restype = c_double
@@ -440,7 +467,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.JointEnsembleProbability.restype = c_double
@@ -448,7 +475,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_int),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.PhaseParity.restype = None
@@ -456,7 +483,7 @@ class QrackSystem:
             c_ulonglong,
             c_double,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.PhaseRootN.restype = None
@@ -464,7 +491,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.ResetAll.restype = None
@@ -513,7 +540,7 @@ class QrackSystem:
             c_ulonglong,
             c_double,
             c_double,
-            c_double
+            c_double,
         ]
 
         self.qrack_lib.Mtrx.restype = None
@@ -526,7 +553,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCY.restype = None
@@ -534,7 +561,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCZ.restype = None
@@ -542,7 +569,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCH.restype = None
@@ -550,7 +577,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCS.restype = None
@@ -558,7 +585,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCT.restype = None
@@ -566,7 +593,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCAdjS.restype = None
@@ -574,7 +601,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCAdjT.restype = None
@@ -582,7 +609,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MCU.restype = None
@@ -593,7 +620,7 @@ class QrackSystem:
             c_ulonglong,
             c_double,
             c_double,
-            c_double
+            c_double,
         ]
 
         self.qrack_lib.MCMtrx.restype = None
@@ -602,7 +629,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             POINTER(c_double),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         # multi-anti-controlled single-qubit gates
@@ -612,7 +639,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACY.restype = None
@@ -620,7 +647,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACZ.restype = None
@@ -628,7 +655,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACH.restype = None
@@ -636,7 +663,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACS.restype = None
@@ -644,7 +671,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACT.restype = None
@@ -652,7 +679,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACAdjS.restype = None
@@ -660,7 +687,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACAdjT.restype = None
@@ -668,7 +695,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.MACU.restype = None
@@ -679,7 +706,7 @@ class QrackSystem:
             c_ulonglong,
             c_double,
             c_double,
-            c_double
+            c_double,
         ]
 
         self.qrack_lib.MACMtrx.restype = None
@@ -688,7 +715,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             POINTER(c_double),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.UCMtrx.restype = None
@@ -698,7 +725,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             POINTER(c_double),
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.Multiplex1Mtrx.restype = None
@@ -707,7 +734,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_double)
+            POINTER(c_double),
         ]
 
         # coalesced single qubit gates
@@ -733,7 +760,7 @@ class QrackSystem:
             c_double,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_ulonglong
+            c_ulonglong,
         ]
 
         # exponential of Pauli operators
@@ -744,7 +771,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_int),
             c_double,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCExp.restype = None
@@ -755,7 +782,7 @@ class QrackSystem:
             c_double,
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         # measurements
@@ -769,12 +796,15 @@ class QrackSystem:
         self.qrack_lib.MAll.restype = c_ulonglong
         self.qrack_lib.MAll.argtypes = [c_ulonglong]
 
+        self.qrack_lib.MAllLong.restype = None
+        self.qrack_lib.MAllLong.argtypes = [c_ulonglong, POINTER(c_ulonglong)]
+
         self.qrack_lib.Measure.restype = c_ulonglong
         self.qrack_lib.Measure.argtypes = [
             c_ulonglong,
             c_ulonglong,
             POINTER(c_int),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MeasureShots.restype = None
@@ -783,7 +813,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         # swap
@@ -803,7 +833,7 @@ class QrackSystem:
             c_double,
             c_double,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.CSWAP.restype = None
@@ -812,7 +842,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.ACSWAP.restype = None
@@ -821,7 +851,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         # Schmidt decomposition
@@ -830,21 +860,21 @@ class QrackSystem:
         self.qrack_lib.Compose.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.Decompose.restype = c_ulonglong
         self.qrack_lib.Decompose.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.Dispose.restype = None
         self.qrack_lib.Dispose.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         # (quasi-)Boolean gates
@@ -854,7 +884,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.OR.restype = None
@@ -862,7 +892,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.XOR.restype = None
@@ -870,7 +900,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.NAND.restype = None
@@ -878,7 +908,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.NOR.restype = None
@@ -886,7 +916,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         self.qrack_lib.XNOR.restype = None
@@ -894,7 +924,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             c_ulonglong,
-            c_ulonglong
+            c_ulonglong,
         ]
 
         # half classical (quasi-)Boolean gates
@@ -933,7 +963,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.SUB.restype = None
@@ -942,7 +972,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.ADDS.restype = None
@@ -952,7 +982,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.SUBS.restype = None
@@ -962,7 +992,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MUL.restype = None
@@ -972,7 +1002,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.DIV.restype = None
@@ -982,7 +1012,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MULN.restype = None
@@ -993,7 +1023,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.DIVN.restype = None
@@ -1004,7 +1034,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.POWN.restype = None
@@ -1015,7 +1045,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCADD.restype = None
@@ -1026,7 +1056,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCSUB.restype = None
@@ -1037,7 +1067,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCMUL.restype = None
@@ -1048,7 +1078,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCDIV.restype = None
@@ -1059,7 +1089,7 @@ class QrackSystem:
             c_ulonglong,
             POINTER(c_ulonglong),
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCMULN.restype = None
@@ -1072,7 +1102,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCDIVN.restype = None
@@ -1085,7 +1115,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.MCPOWN.restype = None
@@ -1098,7 +1128,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.LDA.restype = None
@@ -1108,7 +1138,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ubyte)
+            POINTER(c_ubyte),
         ]
 
         self.qrack_lib.ADC.restype = None
@@ -1119,7 +1149,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ubyte)
+            POINTER(c_ubyte),
         ]
 
         self.qrack_lib.SBC.restype = None
@@ -1130,7 +1160,7 @@ class QrackSystem:
             POINTER(c_ulonglong),
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ubyte)
+            POINTER(c_ubyte),
         ]
 
         self.qrack_lib.Hash.restype = None
@@ -1138,7 +1168,7 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            POINTER(c_ubyte)
+            POINTER(c_ubyte),
         ]
 
         # miscellaneous
@@ -1154,14 +1184,14 @@ class QrackSystem:
             c_ulonglong,
             c_ulonglong,
             POINTER(c_ulonglong),
-            c_double
+            c_double,
         ]
 
         self.qrack_lib.Separate.restype = None
         self.qrack_lib.Separate.argtypes = [
             c_ulonglong,
             c_ulonglong,
-            POINTER(c_ulonglong)
+            POINTER(c_ulonglong),
         ]
 
         self.qrack_lib.GetUnitaryFidelity.restype = c_double
@@ -1195,7 +1225,15 @@ class QrackSystem:
         self.qrack_lib.qstabilizer_in_from_file.argtypes = [c_ulonglong, c_char_p]
 
         self.qrack_lib.init_qneuron.restype = c_ulonglong
-        self.qrack_lib.init_qneuron.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong), c_ulonglong, c_ulonglong, c_double, c_double]
+        self.qrack_lib.init_qneuron.argtypes = [
+            c_ulonglong,
+            c_ulonglong,
+            POINTER(c_ulonglong),
+            c_ulonglong,
+            c_ulonglong,
+            c_double,
+            c_double,
+        ]
 
         self.qrack_lib.clone_qneuron.restype = c_ulonglong
         self.qrack_lib.clone_qneuron.argtypes = [c_ulonglong]
@@ -1210,8 +1248,14 @@ class QrackSystem:
             self.qrack_lib.set_qneuron_angles.argtypes = [c_ulonglong, POINTER(c_float)]
             self.qrack_lib.get_qneuron_angles.argtypes = [c_ulonglong, POINTER(c_float)]
         elif self.fppow == 6:
-            self.qrack_lib.set_qneuron_angles.argtypes = [c_ulonglong, POINTER(c_double)]
-            self.qrack_lib.get_qneuron_angles.argtypes = [c_ulonglong, POINTER(c_double)]
+            self.qrack_lib.set_qneuron_angles.argtypes = [
+                c_ulonglong,
+                POINTER(c_double),
+            ]
+            self.qrack_lib.get_qneuron_angles.argtypes = [
+                c_ulonglong,
+                POINTER(c_double),
+            ]
 
         self.qrack_lib.set_qneuron_alpha.restype = None
         self.qrack_lib.set_qneuron_alpha.argtypes = [c_ulonglong, c_double]
@@ -1238,7 +1282,12 @@ class QrackSystem:
         self.qrack_lib.qneuron_learn.argtypes = [c_ulonglong, c_double, c_bool, c_bool]
 
         self.qrack_lib.qneuron_learn_permutation.restype = None
-        self.qrack_lib.qneuron_learn_permutation.argtypes = [c_ulonglong, c_double, c_bool, c_bool]
+        self.qrack_lib.qneuron_learn_permutation.argtypes = [
+            c_ulonglong,
+            c_double,
+            c_bool,
+            c_bool,
+        ]
 
         self.qrack_lib.init_qcircuit.restype = c_ulonglong
         self.qrack_lib.init_qcircuit.argtypes = [c_bool, c_bool]
@@ -1250,7 +1299,11 @@ class QrackSystem:
         self.qrack_lib.qcircuit_inverse.argtypes = [c_ulonglong]
 
         self.qrack_lib.qcircuit_past_light_cone.restype = c_ulonglong
-        self.qrack_lib.qcircuit_past_light_cone.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong)]
+        self.qrack_lib.qcircuit_past_light_cone.argtypes = [
+            c_ulonglong,
+            c_ulonglong,
+            POINTER(c_ulonglong),
+        ]
 
         self.qrack_lib.destroy_qcircuit.restype = None
         self.qrack_lib.destroy_qcircuit.argtypes = [c_ulonglong]
@@ -1262,10 +1315,21 @@ class QrackSystem:
         self.qrack_lib.qcircuit_swap.argtypes = [c_ulonglong, c_ulonglong, c_ulonglong]
 
         self.qrack_lib.qcircuit_append_1qb.restype = None
-        self.qrack_lib.qcircuit_append_1qb.argtypes = [c_ulonglong, POINTER(c_double), c_ulonglong]
+        self.qrack_lib.qcircuit_append_1qb.argtypes = [
+            c_ulonglong,
+            POINTER(c_double),
+            c_ulonglong,
+        ]
 
         self.qrack_lib.qcircuit_append_mc.restype = None
-        self.qrack_lib.qcircuit_append_mc.argtypes = [c_ulonglong, POINTER(c_double), c_ulonglong, POINTER(c_ulonglong), c_ulonglong, c_ulonglong]
+        self.qrack_lib.qcircuit_append_mc.argtypes = [
+            c_ulonglong,
+            POINTER(c_double),
+            c_ulonglong,
+            POINTER(c_ulonglong),
+            c_ulonglong,
+            c_ulonglong,
+        ]
 
         self.qrack_lib.qcircuit_run.restype = None
         self.qrack_lib.qcircuit_run.argtypes = [c_ulonglong, c_ulonglong]
@@ -1275,3 +1339,9 @@ class QrackSystem:
 
         self.qrack_lib.qcircuit_in_from_file.restype = None
         self.qrack_lib.qcircuit_in_from_file.argtypes = [c_ulonglong, c_char_p]
+
+        self.qrack_lib.qcircuit_out_to_string_length.restype = c_size_t
+        self.qrack_lib.qcircuit_out_to_string_length.argtypes = [c_ulonglong]
+
+        self.qrack_lib.qcircuit_out_to_string.restype = None
+        self.qrack_lib.qcircuit_out_to_string.argtypes = [c_ulonglong, c_char_p]
